package org.processmining.geneticminerusingpartialknowledge.algorithms;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;

import org.deckfour.xes.classification.XEventClass;
import org.processmining.models.graphbased.LocalNodeID;
import org.processmining.models.graphbased.directed.petrinet.Petrinet;
import org.processmining.models.graphbased.directed.petrinet.PetrinetEdge;
import org.processmining.models.graphbased.directed.petrinet.PetrinetNode;
import org.processmining.models.graphbased.directed.petrinet.elements.Place;
import org.processmining.models.graphbased.directed.petrinet.elements.Transition;
import org.processmining.models.heuristics.impl.HNSet;
import org.processmining.models.heuristics.impl.HNSubSet;
import org.processmining.models.heuristics.impl.HeuristicsNetImpl;

public class KnowledgeConverter {
	private ArrayList<TransitionMatch> transitions;
	private TransitionMatch[] transArray;
	private Petrinet PartialKnowledge;

	public TransitionMatch[] getTransArray() {
		if (transArray == null) {
			transArray = transitions.toArray(new TransitionMatch[0]);
		}
		return transArray;
	}

	public KnowledgeConverter(Petrinet partialKnowledge) {
		PartialKnowledge = partialKnowledge;
		transitions = new ArrayList<TransitionMatch>();
	}

	public ArrayList<TransitionMatch> getTransitions() {
		if (transitions != null) {
			return transitions;
		}
		return new ArrayList<TransitionMatch>();
	}

	public TransitionMatch GetTransitionByLocalId(final LocalNodeID localId) {
		TransitionMatch[] transArray = (TransitionMatch[]) transitions.toArray();

		TransitionMatch transition = null;
		for (int i = 0; i < transArray.length; i++) {
			if (transArray[i].getPetriNetNodeLocalId() == localId) {
				transition = transArray[i];
				break;
			}
		}

		if (transition == null) {
			System.out.println("Did not find matching transition");
			return null;
		}
		return transition;

	}

	public void addTransition(TransitionMatch transition) {
		if (getTransitions().contains(transition)) {
			System.out.println(transition + " already matched");
			return;
		}
		getTransitions().add(transition);
	}

	public void generateTransitionMatching(Collection<Transition> petriTransitions,
			XEventClass[] heuristicTransitions) {

		for (Iterator<Transition> iterator = petriTransitions.iterator(); iterator.hasNext();) {
			final Transition transition = iterator.next();
			String fixedTransitionLabel = FixLabel(transition.getLabel());
			for (int i = 0; i < heuristicTransitions.length; i++) {
				System.out.println("...generateTransitionMatching looking for transition,compare: "
						+ heuristicTransitions[i].getId() + " and " + fixedTransitionLabel);
				if (heuristicTransitions[i].getId().equals(fixedTransitionLabel)) {
					addTransition(new TransitionMatch(transition.getLabel(), transition.getLocalID(),
							heuristicTransitions[i].getId(), heuristicTransitions[i].getIndex()));
				}
			}
		}
		PrintTransitions();
	}

	public String FixLabel(String label) {

		String returnLabel = label;
		System.out.println("...Fixlabel param: " + label);
		if (!label.contains("complete") && !label.contains("start")) {
			returnLabel = returnLabel.replaceFirst("\\+", "\\+complete");
		} else if (label.contains("+ ")) {
			returnLabel = returnLabel.replaceFirst("\\+ ", "\\+");
		}

		System.out.println("...Fixlabel return: " + returnLabel);
		return returnLabel;
	}

	public void PrintTransitions() {
		System.out.println("Current 'transitions' state: ");
		for (TransitionMatch match : transitions) {
			System.out.println(match);
		}
	}

	public HeuristicsNetImpl MapPlacesToHNAInOutSets(Collection<Place> pnPlaces,
			HeuristicsNetImpl heuristicPopulation) {
		if (transitions == null) {
			System.err.println("Transition matching has to be generated.");
		}

		for (Iterator<Place> iterator = pnPlaces.iterator(); iterator.hasNext();) {
			Place place = iterator.next();

			Collection<PetrinetEdge<? extends PetrinetNode, ? extends PetrinetNode>> inEdges = place.getGraph()
					.getInEdges(place);
			Collection<PetrinetEdge<? extends PetrinetNode, ? extends PetrinetNode>> outEdges = place.getGraph()
					.getOutEdges(place);

			Collection<TransitionMatch> srcTransitions = MapEdgesToTranstitions(inEdges, EdgeType.IN);
			Collection<TransitionMatch> trgTransitions = MapEdgesToTranstitions(outEdges, EdgeType.OUT);

			HNSubSet srcSubSet = GetSubSetFromTransitions(srcTransitions);
			HNSet srcSet = new HNSet();
			srcSet.add(srcSubSet);

			HNSubSet trgSubSet = GetSubSetFromTransitions(trgTransitions);
			HNSet trgSet = new HNSet();
			trgSet.add(trgSubSet);

			for (Iterator iterator2 = srcTransitions.iterator(); iterator2.hasNext();) {
				TransitionMatch transitionMatch = (TransitionMatch) iterator2.next();
				HNSet currentSet = heuristicPopulation.getOutputSet(transitionMatch.getHeuristicIndex());
				if (currentSet != null) {
					trgSet.addAll(currentSet);
				}
				heuristicPopulation.setOutputSet(transitionMatch.getHeuristicIndex(), trgSet);
			}

			for (Iterator iterator3 = trgTransitions.iterator(); iterator3.hasNext();) {
				TransitionMatch transitionMatch = (TransitionMatch) iterator3.next();
				HNSet currentSet = heuristicPopulation.getInputSet(transitionMatch.getHeuristicIndex());
				if (currentSet != null) {
					srcSet.addAll(currentSet);
				}
				heuristicPopulation.setInputSet(transitionMatch.getHeuristicIndex(), srcSet);
			}

		}
		FillNullSubsets(heuristicPopulation);
		return heuristicPopulation;
	}

	private void FillNullSubsets(HeuristicsNetImpl heuristicPopulation) {
		int inputSetsLength = heuristicPopulation.getInputSets().length;
		for (int i = 0; i < inputSetsLength; i++) {
			if (heuristicPopulation.getInputSet(i) == null) {
				heuristicPopulation.setInputSet(i, new HNSet());
			}
		}
		int outputSetsLength = heuristicPopulation.getOutputSets().length;
		for (int i = 0; i < outputSetsLength; i++) {
			if (heuristicPopulation.getOutputSet(i) == null) {
				heuristicPopulation.setOutputSet(i, new HNSet());
			}
		}

	}

	private HNSubSet GetSubSetFromTransitions(Collection<TransitionMatch> transitionsCollection) {
		HNSubSet subSet = new HNSubSet();

		for (Iterator iterator = transitionsCollection.iterator(); iterator.hasNext();) {
			TransitionMatch transition = (TransitionMatch) iterator.next();
			subSet.add(transition.getHeuristicIndex());
		}
		return subSet;
	}

	private ArrayList<TransitionMatch> MapEdgesToTranstitions(
			Collection<PetrinetEdge<? extends PetrinetNode, ? extends PetrinetNode>> inEdges, EdgeType edge) {

		ArrayList<TransitionMatch> transitionsMapped = new ArrayList<TransitionMatch>();

		for (Iterator iterator = inEdges.iterator(); iterator.hasNext();) {
			PetrinetEdge<? extends PetrinetNode, ? extends PetrinetNode> petrinetEdge = (PetrinetEdge<? extends PetrinetNode, ? extends PetrinetNode>) iterator
					.next();
			final LocalNodeID localId;
			if (edge == EdgeType.IN) {
				localId = petrinetEdge.getSource().getLocalID();
			} else {
				localId = petrinetEdge.getTarget().getLocalID();
			}

			TransitionMatch[] array = getTransArray();

			TransitionMatch elem = null;
			for (int i = 0; i < array.length; i++) {
				if (array[i].getPetriNetNodeLocalId() == localId) {
					elem = array[i];
					break;
				}
			}

			if (elem != null) {
				transitionsMapped.add(elem);
			}

		}
		return transitionsMapped;
	}

	public HeuristicsNetImpl GenerateInitialPopulationWithPartialKnowledge(HeuristicsNetImpl heuristicPopulation,
			XEventClass[] hnaTransitions) {
		//get used transitions  from PetriNet
		Collection<Place> pnPlaces = PartialKnowledge.getPlaces();
		Collection<Transition> pnTransitions = PartialKnowledge.getTransitions();
		//map transitions to ids of ams activities
		generateTransitionMatching(pnTransitions, hnaTransitions);

		//map places to input/output sets of heuristicNetArray	
		return MapPlacesToHNAInOutSets(pnPlaces, heuristicPopulation);
	}
}
