package org.processmining.geneticminerusingpartialknowledge.algorithms;

import java.util.Random;

import javax.swing.JOptionPane;

import org.deckfour.xes.info.XLogInfo;
import org.processmining.contexts.uitopia.UIPluginContext;
import org.processmining.framework.connections.ConnectionCannotBeObtained;
import org.processmining.models.graphbased.directed.petrinet.Petrinet;
import org.processmining.models.heuristics.HeuristicsNet;
import org.processmining.models.heuristics.impl.HNSet;
import org.processmining.models.heuristics.impl.HeuristicsNetImpl;
import org.processmining.models.semantics.petrinet.Marking;
import org.processmining.plugins.heuristicsnet.miner.genetic.fitness.Fitness;
import org.processmining.plugins.heuristicsnet.miner.genetic.fitness.FitnessFactory;
import org.processmining.plugins.heuristicsnet.miner.genetic.miner.settings.GeneticMinerSettings;
import org.processmining.plugins.heuristicsnet.miner.genetic.population.InitialPopulationFactory;
import org.processmining.plugins.heuristicsnet.miner.heuristics.converter.HeuristicsNetToPetriNetConverter;
import org.processmining.plugins.petrinet.reduction.Murata;

public class PartialKnowledgeHelper {
	public static Fitness GetFitness(int populationSize, XLogInfo logInfo, GeneticMinerSettings settings) {
		settings.setPopulationSize(populationSize);
		Fitness fitness = FitnessFactory.getFitness(settings.getFitnessType(), logInfo,
				FitnessFactory.ALL_FITNESS_PARAMETERS);
		return fitness;
	}

	public static int GetPopulationSize() {
		String input = (String)JOptionPane.showInputDialog(null,
				"Population size to generate:", "Please enter new quantity",
				JOptionPane.QUESTION_MESSAGE,null,null,"10");

		System.out.println("value: " + input);

		int populationSize = Integer.parseInt(input);
		return populationSize;
	}


	public static HeuristicsNet[] MergeInputOutputSets(HeuristicsNet[] baseGeneration, HeuristicsNetImpl toMergeIn,Fitness fitness)
	{
		System.out.println("Mergin input/output sets");
		HeuristicsNet[] editedGeneration = new HeuristicsNet[baseGeneration.length];

		for (int t = 0; t< baseGeneration.length; t++){
			editedGeneration[t] = new HeuristicsNetImpl(toMergeIn.getActivitiesMappingStructures());
			//dodaj do edited wszystkie input z base
			editedGeneration[t] = AddAllInputSets(baseGeneration[t], editedGeneration[t]);

			//dodaj inteligentnie sety z outputu
			//editedGeneration[t] = InteligentAddOutputSets(baseGeneration[t].getOutputSets(),editedGeneration[t],probabiltyOfRemoval);
			editedGeneration[t] = AddAllOutputSets(baseGeneration[t], editedGeneration[t]);

			//dodaj wszystko z wiedzy poczatkowej
			editedGeneration[t] = AddAllInputSets(toMergeIn, editedGeneration[t]);
			editedGeneration[t] = AddAllOutputSets(toMergeIn, editedGeneration[t]);

		}
		editedGeneration = fitness.calculate(editedGeneration);
		//editedGeneration = MethodsOverHeuristicsNets.removeUnusedElements(editedGeneration, fitness);
		System.out.println("Merge done");
		return editedGeneration;
	}


	public static HeuristicsNet AddAllInputSets(HeuristicsNet baseSpecimen, HeuristicsNet editedSpecimen) {
		for (int y = 0; y< baseSpecimen.getInputSets().length; y++){				
			HNSet newset = new HNSet();
			newset = AddSetIfNotNull(newset, baseSpecimen.getInputSet(y));
			newset = AddSetIfNotNull(newset, editedSpecimen.getInputSet(y));
			editedSpecimen.setInputSet(y, newset);
		}
		return editedSpecimen;
	}

	public static HNSet AddSetIfNotNull(HNSet addHereSet, HNSet setToAdd){
		if(setToAdd != null)
		{
			addHereSet.addAll(setToAdd);
			return addHereSet;
		}
		return addHereSet;
	}

	public static HeuristicsNet AddAllOutputSets(HeuristicsNet baseSpecimen, HeuristicsNet editedSpecimen) {
		for (int y = 0; y< baseSpecimen.getOutputSets().length; y++){				
			HNSet newset = new HNSet();
			newset = AddSetIfNotNull(newset, baseSpecimen.getOutputSet(y));
			newset = AddSetIfNotNull(newset, editedSpecimen.getOutputSet(y));
			editedSpecimen.setOutputSet(y, newset);
		}
		return editedSpecimen;
	}

	public static HeuristicsNet[] GenerateRandomPopulation(XLogInfo logInfo, GeneticMinerSettings settings) {
		Random generator = new Random(settings.getSeed());
		HeuristicsNet[] randomPopulation = new HeuristicsNet[settings.getPopulationSize()];
		return InitialPopulationFactory
				.getPopulation(settings.getInitialPopulationType(), generator, logInfo, settings.getPower())
				.build(randomPopulation);
	}
	
	public static HeuristicsNet[] FillOutPopulation( HeuristicsNet[] population,
			HeuristicsNetImpl heuristicPopulation) {
		for (int i = 0; i < population.length; i++) {
			population[i] = heuristicPopulation;
		}
		return population;
	}

	public static Object[] ConvertToPetrinet(UIPluginContext context, HeuristicsNet heuristicsNet) {
		
		HeuristicsNetToPetriNetConverter HNconverter = new HeuristicsNetToPetriNetConverter();
		
		Object[] convertedPetrinet = HNconverter.converter(context,heuristicsNet);
		Murata murata= new Murata();
		try {
			return murata.run(context, (Petrinet)convertedPetrinet[0], (Marking)convertedPetrinet[1]);
		} catch (ConnectionCannotBeObtained e) {
			context.log("Failed to convert from HeuristicNet to Petrinet");
			context.log(e);
			e.printStackTrace();
		}
		return null;
	}
	
	public static HeuristicsNet GetBestSpecimen(HeuristicsNet[] population)
	{
		HeuristicsNet best = population[0];
		for(int i = 1; i < population.length; i++)
		{
			double tmpfitness = population[i].getFitness();
			if(tmpfitness > best.getFitness() )
			{
				best = population[i];
			}
		}
		return best;
	}
}
