package org.processmining.geneticminerusingpartialknowledge.plugins;


import org.deckfour.xes.classification.XEventClass;
import org.deckfour.xes.classification.XEventClasses;
import org.deckfour.xes.info.XLogInfo;
import org.deckfour.xes.info.XLogInfoFactory;
import org.deckfour.xes.model.XLog;
import org.processmining.contexts.uitopia.UIPluginContext;
import org.processmining.contexts.uitopia.annotations.UITopiaVariant;
import org.processmining.framework.plugin.annotations.Plugin;
import org.processmining.geneticminerusingpartialknowledge.algorithms.KnowledgeConverter;
import org.processmining.geneticminerusingpartialknowledge.algorithms.MeasurmentLogger;
import org.processmining.geneticminerusingpartialknowledge.algorithms.PartialKnowledgeHelper;
import org.processmining.geneticminerusingpartialknowledge.algorithms.TimeMeasure;
import org.processmining.models.graphbased.directed.petrinet.Petrinet;
import org.processmining.models.heuristics.HeuristicsNet;
import org.processmining.models.heuristics.impl.ActivitiesMappingStructures;
import org.processmining.models.heuristics.impl.HeuristicsNetImpl;
import org.processmining.models.semantics.petrinet.Marking;
import org.processmining.plugins.heuristicsnet.array.visualization.HeuristicsNetArrayObject;
import org.processmining.plugins.heuristicsnet.miner.genetic.fitness.Fitness;
import org.processmining.plugins.heuristicsnet.miner.genetic.miner.GeneticMiner;
import org.processmining.plugins.heuristicsnet.miner.genetic.miner.GeneticMinerPlugin4UITopia;
import org.processmining.plugins.heuristicsnet.miner.genetic.miner.settings.GeneticMinerSettings;

public class GMFPKMeasurmentFriendly {
	public static TimeMeasure TimeMeasure = new TimeMeasure();

	@UITopiaVariant(affiliation = "Gdansk University of Technology", author = "Beata Legowska", email = "beata.legowska@gmail.com")
	@Plugin(name = "AAAAA Genetic Miner using partial knowledge - measurement version", 
		parameterLabels = { "Log", "Knowledge"}, 
		returnLabels = {"Petri net reduced random", "Marking random", "Petri net reduced edited", 
						"Marking edited", "Random Heuristics population", "Edited Heurists population"},
	returnTypes = {Petrinet.class, Marking.class, Petrinet.class, Marking.class, HeuristicsNetArrayObject.class, HeuristicsNetArrayObject.class}, userAccessible = false, help = "Produces first iteration population for Genetic Algorithm")

	public static  Object[] helloWorlds(UIPluginContext context, XLog log, Petrinet pnKnowledge) {
		
		HeuristicsNetArrayObject editedResult = new HeuristicsNetArrayObject(null);
		HeuristicsNetArrayObject randomResult = new HeuristicsNetArrayObject(null);	
		//creation of HeuricticsNet out of log info, basic data about transitions/activities only, no places/arcs included
		XLogInfo logInfo = XLogInfoFactory.createLogInfo(log);
		XEventClasses events = logInfo.getEventClasses();
		ActivitiesMappingStructures ams = new ActivitiesMappingStructures(events);

		for (int populationSize = 10; populationSize<12; populationSize=populationSize*2)
		{
			HeuristicsNet[] population = new HeuristicsNetImpl[populationSize];
			HeuristicsNetImpl heuristicPopulation = new HeuristicsNetImpl(ams);
			XEventClass[] hnaTransitions = heuristicPopulation.getActivitiesMappingStructures().getActivitiesMapping();

			KnowledgeConverter converter = new KnowledgeConverter(pnKnowledge);	
			
			heuristicPopulation = converter.GenerateInitialPopulationWithPartialKnowledge(heuristicPopulation, hnaTransitions);

			MeasurmentLogger loggerShort = new MeasurmentLogger(log.size(), pnKnowledge ,"Random initial population,Improved initial population, Probability of removing random set, Population size:"+populationSize, "short");
			MeasurmentLogger loggerFull = new MeasurmentLogger(log.size(), pnKnowledge, "Random initial population,Improved initial population, Probability of removing random set,Population size:"+populationSize, "full");

			GeneticMinerSettings settings = new GeneticMinerSettings();
			Fitness fitness = PartialKnowledgeHelper.GetFitness(populationSize, logInfo, settings);

			population = PartialKnowledgeHelper.FillOutPopulation( population, heuristicPopulation);
			population = fitness.calculate(population);
			//from population
			GeneticMiner GMobject = new GeneticMiner(context, logInfo, settings);

			long starttime = TimeMeasure.StartTimeMeasurement();

			for(int k = 0;k < 1; k++)
			{
				System.out.println( " run: " + k + "time passed so far: " + TimeMeasure.EndTimeMeasurement(starttime));
				HeuristicsNet[] randomGeneration = PartialKnowledgeHelper.GenerateRandomPopulation(logInfo, settings);
				randomGeneration = fitness.calculate(randomGeneration);
				HeuristicsNet[] editedGeneration = PartialKnowledgeHelper.MergeInputOutputSets(randomGeneration, heuristicPopulation, fitness);
				editedGeneration = fitness.calculate(editedGeneration);

				System.out.println("running random");
				randomResult = GeneticMinerPlugin4UITopia.GeneticMinerSplitTheLog(context, log, randomGeneration);
				System.out.println("running edited");
				editedResult = GeneticMinerPlugin4UITopia.GeneticMinerSplitTheLog(context, log, editedGeneration);
				
				loggerShort.AddLine(randomResult.getPopulation()[populationSize-1].getFitness()+","+editedResult.getPopulation()[populationSize-1].getFitness()+ ","+populationSize);
			}
		}
		
		Object[] randomPetri = PartialKnowledgeHelper.ConvertToPetrinet(context, PartialKnowledgeHelper.GetBestSpecimen(randomResult.getPopulation()));
		Object[] editedPetri = PartialKnowledgeHelper.ConvertToPetrinet(context, PartialKnowledgeHelper.GetBestSpecimen(editedResult.getPopulation()));		

		return  new Object[] {randomPetri[0], randomPetri[1], editedPetri[0], editedPetri[1], randomResult, editedResult}; 
	}

}
