package org.processmining.geneticminerusingpartialknowledge.plugins;
import java.util.Random;
import org.deckfour.xes.classification.XEventClass;
import org.deckfour.xes.classification.XEventClasses;
import org.deckfour.xes.info.XLogInfo;
import org.deckfour.xes.info.XLogInfoFactory;
import org.deckfour.xes.model.XLog;
import org.processmining.contexts.uitopia.UIPluginContext;
import org.processmining.contexts.uitopia.annotations.UITopiaVariant;
import org.processmining.framework.plugin.annotations.Plugin;
import org.processmining.geneticminerusingpartialknowledge.algorithms.KnowledgeConverter;
import org.processmining.geneticminerusingpartialknowledge.algorithms.PartialKnowledgeHelper;
import org.processmining.geneticminerusingpartialknowledge.algorithms.TimeMeasure;
import org.processmining.models.graphbased.directed.petrinet.Petrinet;
import org.processmining.models.heuristics.HeuristicsNet;
import org.processmining.models.heuristics.impl.ActivitiesMappingStructures;
import org.processmining.models.heuristics.impl.HeuristicsNetImpl;
import org.processmining.models.semantics.petrinet.Marking;
import org.processmining.plugins.heuristicsnet.array.visualization.HeuristicsNetArrayObject;
import org.processmining.plugins.heuristicsnet.miner.genetic.fitness.Fitness;
import org.processmining.plugins.heuristicsnet.miner.genetic.miner.GeneticMinerPlugin4UITopia;
import org.processmining.plugins.heuristicsnet.miner.genetic.miner.settings.GeneticMinerSettings;

public class GeneticMinerFromPartialKnowledge {
	public static TimeMeasure TimeMeasure = new TimeMeasure();

	@UITopiaVariant(affiliation = "Gdansk University of Technology", author = "Beata Legowska", email = "beata.legowska@gmail.com")

	@Plugin(name = "Genetic Miner using partial knowledge", parameterLabels = { "Log", "Knowledge" },		returnLabels = {"Petri net", "Marking", "Heuristics population", "Heuristics net - best specimen"},
	returnTypes = {Petrinet.class, Marking.class, HeuristicsNetArrayObject.class, HeuristicsNetImpl.class }, userAccessible = true, help = "Runs Genetic Miner using initial population created based on patrial expert knowledge")

	public static  Object[] GeneticFromPartialKnowledge(UIPluginContext context, XLog log, Petrinet pnKnowledge) {

		int populationSize = PartialKnowledgeHelper.GetPopulationSize();

		HeuristicsNetArrayObject editedResult = new HeuristicsNetArrayObject(null);			

		//creation of HeuricticsNet out of log info, basic data about transitions/activities only, no places/arcs included
		XLogInfo logInfo = XLogInfoFactory.createLogInfo(log);
		XEventClasses events = logInfo.getEventClasses();
		ActivitiesMappingStructures ams = new ActivitiesMappingStructures(events);

		HeuristicsNet[] population = new HeuristicsNetImpl[populationSize];
		HeuristicsNetImpl heuristicPopulation = new HeuristicsNetImpl(ams);

		XEventClass[] hnaTransitions = heuristicPopulation.getActivitiesMappingStructures().getActivitiesMapping();
	
		KnowledgeConverter converter = new KnowledgeConverter(pnKnowledge);	
	
		heuristicPopulation = converter.GenerateInitialPopulationWithPartialKnowledge(heuristicPopulation, hnaTransitions);

		GeneticMinerSettings settings = new GeneticMinerSettings();
		settings.setSeed(new Random().nextLong());
		Fitness fitness = PartialKnowledgeHelper.GetFitness(populationSize, logInfo, settings);

		PartialKnowledgeHelper.FillOutPopulation(population, heuristicPopulation);
		population = fitness.calculate(population);

		HeuristicsNet[] randomGeneration = PartialKnowledgeHelper.GenerateRandomPopulation(logInfo, settings);
		HeuristicsNet[] editedGeneration = PartialKnowledgeHelper.MergeInputOutputSets(randomGeneration, heuristicPopulation, fitness);
		editedGeneration = fitness.calculate(editedGeneration);
		editedResult = GeneticMinerPlugin4UITopia.GeneticMinerSplitTheLog(context, log, editedGeneration);
				
		HeuristicsNet bestSpecimen = PartialKnowledgeHelper.GetBestSpecimen(editedResult.getPopulation());	
		System.out.println("Fitness of generated model: " + bestSpecimen.getFitness() );
	
		Object[] petrinet = PartialKnowledgeHelper.ConvertToPetrinet(context, bestSpecimen);
		
		return new Object[]{petrinet[0], petrinet[1], editedResult, bestSpecimen};
	}

}
